'use strict';

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(function () {
	/**
  * Match non-context columns.
  * @param {Column} _column The column to check
  * @returns {boolean} `false` if the column has type `'context'`, `true` otherwise.
  */
	function _nonContext(_column) {
		return _column.kind !== 'context';
	}

	function _sameArray(arr1, arr2) {
		if (arr1 && arr1.length > 0 && arr2 && arr2.length > 0 && arr1.length === arr2.length) {
			for (var i = 0; i < arr2.length; i++) {
				if (arr1[i] !== arr2[i]) {
					return false;
				}
			}
		} else {
			return false;
		}
		return true;
	}

	return {
		/**
   * Translates a Dashboard aggregation type to a supported aggregation type for this module.
   * @param {string} _aggregation The aggregation as defined by Dashboard.
   * @returns {string} The aggregation supported by this module.
   */
		toSupportedAggregation: function toSupportedAggregation(_aggregation) {
			switch (_aggregation) {
				case 'avg':
				case 'sum':
					return _aggregation;
				default:
					return 'sum'; // Fall back to sum
			}
		},
		/**
   * This function changes the rectangle of data by the variableValues value of each column.
   * the change for measure columns are in percentage. For cat columns, if the variableValues is null,
   * the values are kept as it, o.w. we change all values to the value in variableValues
   * For example, if rectangle of data is [ [10,10, '0'],[20,50, '1']]:
   * CASE1 : variableValues is [0,-50, '-1'] --> result: [[10,5, '0'],[20,25, '1']]
   * CASE1 : variableValues is [100,-50, '1'] --> result: [[20,5, '1'],[40,25, '1']]
   */
		changingColumnBy: function changingColumnBy(data, inputColumns, variableValues) {
			return data.map(function (row) {
				return row.map(function (_sourceValue, colIndex) {
					// If the delta value is `null`, the existing value should be used.
					if (variableValues[colIndex] === null) {
						return _sourceValue;
					}
					// For measures, adjust the values by the provided offset.
					if (inputColumns[colIndex].type === 'measure') {
						var ratio = variableValues[colIndex] + 1;
						return _sourceValue * ratio;
					} else {
						// For categorical, return variableValues
						return variableValues[colIndex];
					}
				});
			});
		},

		/*
   * This function checks whether stored model
   * is applicable to the data and the model version
   * and can be used
   */
		sameModelMetadata: function sameModelMetadata(_metadata1, _metadata2) {
			if (_metadata1 && _metadata2) {
				if (_metadata1.modelVersion !== _metadata2.modelVersion) return false;
				if (_metadata1.targetColId !== _metadata2.targetColId) return false;
				if (_metadata1.targetColType !== _metadata2.targetColType) return false;
				return _sameArray(_metadata1.inputColTypes, _metadata2.inputColTypes) && _sameArray(_metadata1.inputColIds, _metadata2.inputColIds);
			}
			// if one or more metaData is not defined, retuen false
			return false;
		},
		/*
   * This function checks whether new data has same shape as old sourceTable (same set and
   * order of columns), except for the context slot
   */
		sourceTableSameShape: function sourceTableSameShape(_sourceTable, _previousTable) {
			// Ignore context columns in comparison
			var columns = _sourceTable.columns.filter(_nonContext);
			var previousColumns = _previousTable.columns.filter(_nonContext);
			if (columns.length !== previousColumns.length) {
				return false;
			}
			for (var i = 0; i < columns.length; i++) {
				if (columns[i].id !== previousColumns[i].id || columns[i].type !== previousColumns[i].type) {
					return false;
				}
			}
			return true;
		},

		/**
  * Create a percentage formatter.
  * @param {string|undefined} _locale The locale to use for the formatter.
  * @returns {Intl.NumberFormat}
  */
		createPercentFormatter: function createPercentFormatter(_locale) {
			if (typeof Intl !== 'undefined' && typeof Intl.NumberFormat !== 'undefined') {
				return new Intl.NumberFormat(_locale, {
					style: 'percent',
					minimumFractionDigits: 0,
					maximumFractionDigits: 1
				});
			} else {
				return {
					format: function format(_value) {
						return Math.round(_value * 1000) / 10 + '%';
					}
				};
			}
		},

		/**
   * Create a compact number formatter.
   * @param {string|undefined} _locale The locale to use for the formatter.
   * @returns {NumberFormatter}
   */
		createNumberFormatter: function createNumberFormatter(_locale) {
			return new Intl.NumberFormat(_locale ? _locale : 'en-US', { notation: 'compact', maximumSignificantDigits: 2 });
		},

		/**
    * Resolves tuple indices to a string value for the model.
    * @param {number} _value The value to resolve.
    * @param {Column} _column The Column to resolve the value for.
    */
		valueForModel: function valueForModel(_value, _column) {
			return _column.type === 'measure' ? _value : String(_value);
		}

	};
});
//# sourceMappingURL=ScenarioUtils.js.map